@@ -13,8 +13,8 @@
 *
 *  You should have received a copy of the GNU General Public License
 *  along with with program; see the file COPYING. If not, write to the
-*  Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-*  MA  02111-1307  USA
+*  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+*  MA  02110-1301  USA
 */
 
 /**
@@ -161,7 +161,12 @@ main( int argc, char *argv[] )
   }
 
   XLALDestroyCOMPLEX8FFTPlan( prev );
+  XLALDestroyCOMPLEX8FFTPlan( pfwd );
 
+  /* Null pointers should be a no-op */
+  prev = NULL;
+  pfwd = NULL;
+  XLALDestroyCOMPLEX8FFTPlan( prev );
   XLALDestroyCOMPLEX8FFTPlan( pfwd );
 
   LALCDestroyVector( &status, &cvec );
@@ -212,7 +217,7 @@ TestStatus( LALStatus *status, const char *ignored, int exitcode )
     }
     else
     {
-      if ( status->statusCode == atoi( tok ) )
+      if ( status->statusCode == atoi( str ) )
       {
         return;
       }
