@@ -13,8 +13,8 @@
 *
 *  You should have received a copy of the GNU General Public License
 *  along with with program; see the file COPYING. If not, write to the
-*  Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-*  MA  02111-1307  USA
+*  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+*  MA  02110-1301  USA
 */
 
 /**
@@ -301,6 +301,13 @@ int main( int argc, char *argv[] )
     TestStatus( &status, CODES( 0 ), 1 );
     XLALDestroyREAL4FFTPlan( fwd );
     XLALDestroyREAL4FFTPlan( rev );
+
+    /* Null pointers should be a no-op */
+    rev = NULL;
+    fwd = NULL;
+    XLALDestroyREAL4FFTPlan( fwd );
+    XLALDestroyREAL4FFTPlan( rev );
+
     TestStatus( &status, CODES( 0 ), 1 );
   }
 
@@ -342,7 +349,7 @@ TestStatus( LALStatus *status, const char *ignored, int exitcode )
     }
     else
     {
-      if ( status->statusCode == atoi( tok ) )
+      if ( status->statusCode == atoi( str ) )
       {
         return;
       }
