@@ -13,8 +13,8 @@
 *
 *  You should have received a copy of the GNU General Public License
 *  along with with program; see the file COPYING. If not, write to the
-*  Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-*  MA  02111-1307  USA
+*  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+*  MA  02110-1301  USA
 */
 
 /**
@@ -192,10 +192,6 @@ int main (int argc, char *argv[])
   TestStatus (&status, CODES(0), 1);
 
 
-#ifndef LAL_NDEBUG
-
-  if ( ! lalNoDebug )
-  {
     if ( verbose )
       printf ("\nCheck error conditions:\n");
 
@@ -253,7 +249,7 @@ int main (int argc, char *argv[])
     if (xlalErrno == XLAL_ESIZE)
       xlalErrno = 0;
     else
-      abort();
+      XLAL_ERROR_MAIN(xlalErrno, "xlalErrno=%i (%s) at line %i", xlalErrno, XLALErrorString(xlalErrno), __LINE__);
     if ( verbose )
       printf ("Invalid size check passed.\n");
 
@@ -269,11 +265,9 @@ int main (int argc, char *argv[])
     if (xlalErrno == XLAL_EFPDIV0)
       xlalErrno = 0;
     else
-      abort();
+      XLAL_ERROR_MAIN(xlalErrno, "xlalErrno=%i (%s) at line %i", xlalErrno, XLALErrorString(xlalErrno), __LINE__);
     if ( verbose )
       printf ("Zero divide check passed.\n");
-  }
-#endif
 
 
   LALSDestroyVector (&status, &x);
@@ -323,7 +317,7 @@ TestStatus (LALStatus *status, const char *ignored, int exitcode)
     }
     else
     {
-      if (status->statusCode == atoi (tok))
+      if (status->statusCode == atoi (str))
       {
         return;
       }
