@@ -13,8 +13,8 @@
 *
 *  You should have received a copy of the GNU General Public License
 *  along with with program; see the file COPYING. If not, write to the
-*  Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-*  MA  02111-1307  USA
+*  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+*  MA  02110-1301  USA
 */
 #include <config.h>
 
@@ -180,9 +180,6 @@ main (int argc, char *argv[])
    */
 
 
-#ifndef LAL_NDEBUG
-  if ( ! lalNoDebug )
-  {
     if (verbose || lalDebugLevel)
     {
       printf ("\n===== Check Errors =====\n");
@@ -203,8 +200,6 @@ main (int argc, char *argv[])
         exit (1);
       XLALClearErrno();
     }
-  }
-#endif
 
 
   /*
@@ -263,7 +258,7 @@ TestStatus (LALStatus *status, const char *ignored, int exitcode)
     }
     else
     {
-      if (status->statusCode == atoi (tok))
+      if (status->statusCode == atoi (str))
       {
         return;
       }
