@@ -13,8 +13,8 @@
 *
 *  You should have received a copy of the GNU General Public License
 *  along with with program; see the file COPYING. If not, write to the
-*  Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-*  MA  02111-1307  USA
+*  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+*  MA  02110-1301  USA
 */
 #include <math.h>
 #include <stdlib.h>
@@ -39,6 +39,40 @@
 #define Relerrd(dx,x) (fabs(x)>0 ? fabs((dx)/(x)) : fabs(dx) )
 #define cRelerr(dx,x) (cabsf(x)>0 ? cabsf((dx)/(x)) : fabsf(dx) )
 
+// ----- test and benchmark operators with 1 REAL4 vector input and 1 INT4 vector output (S2I) ----------
+#define TESTBENCH_VECTORMATH_S2I(name,in)                               \
+  {                                                                     \
+    XLAL_CHECK ( XLALVector##name##REAL4_GEN( xOutRefI4->data, in, Ntrials ) == XLAL_SUCCESS, XLAL_EFUNC ); \
+    tic = XLALGetCPUTime();                                             \
+    for (UINT4 l=0; l < Nruns; l ++ ) {                                 \
+      XLAL_CHECK ( XLALVector##name##REAL4( xOutI4->data, in, Ntrials ) == XLAL_SUCCESS, XLAL_EFUNC ); \
+    }                                                                   \
+    toc = XLALGetCPUTime();                                             \
+    for ( UINT4 i = 0; i < Ntrials; i ++ )                              \
+    {                                                                   \
+      XLAL_CHECK ( xOutI4->data[i] == xOutRefI4->data[i], XLAL_ETOL, "%s: found element #%u (%i) differs from reference (%i)", #name, i, xOutI4->data[i], xOutRefI4->data[i] ); \
+    }                                                                   \
+    XLALPrintInfo ( "%-32s: %4.0f Mops/sec\n", XLALVector##name##REAL4_name, (REAL8)Ntrials * Nruns / (toc - tic)/1e6 ); \
+  }
+
+// ----- test and benchmark operators with 1 REAL4 vector input and 1 REAL4 scalar output (S2s) ----------
+#define TESTBENCH_VECTORMATH_S2s(name,in)                               \
+  {                                                                     \
+    REAL4 sOutRef, sOut;                                                \
+    XLAL_CHECK ( XLALVector##name##REAL4_GEN( &sOutRef, in, Ntrials ) == XLAL_SUCCESS, XLAL_EFUNC ); \
+    tic = XLALGetCPUTime();                                           \
+    for (UINT4 l=0; l < Nruns; l ++ ) {                                 \
+      XLAL_CHECK ( XLALVector##name##REAL4( &sOut, in, Ntrials ) == XLAL_SUCCESS, XLAL_EFUNC ); \
+    }                                                                   \
+    toc = XLALGetCPUTime();                                           \
+    maxErr = fabsf( sOut - sOutRef );                                   \
+    maxRelerr = Relerr( maxErr, sOutRef );                                \
+    XLALPrintInfo ( "%-32s: %4.0f Mops/sec [maxErr = %7.2g (tol=%7.2g), maxRelerr = %7.2g (tol=%7.2g)]\n", \
+                    XLALVector##name##REAL4_name, (REAL8)Ntrials * Nruns / (toc - tic)/1e6, maxErr, (abstol), maxRelerr, (reltol) ); \
+    XLAL_CHECK ( (maxErr <= (abstol)), XLAL_ETOL, "%s: absolute error (%g) exceeds tolerance (%g)\n", #name "REAL4", maxErr, abstol ); \
+    XLAL_CHECK ( (maxRelerr <= (reltol)), XLAL_ETOL, "%s: relative error (%g) exceeds tolerance (%g)\n", #name "REAL4", maxRelerr, reltol ); \
+  }
+
 // ----- test and benchmark operators with 1 REAL4 vector input and 1 REAL4 vector output (S2S) ----------
 #define TESTBENCH_VECTORMATH_S2S(name,in)                               \
   {                                                                     \
@@ -62,6 +96,7 @@
     XLAL_CHECK ( (maxRelerr <= (reltol)), XLAL_ETOL, "%s: relative error (%g) exceeds tolerance (%g)\n", #name "REAL4", maxRelerr, reltol ); \
   }
 
+// ----- test and benchmark operators with 1 REAL4 vector input to 2 REAL4 vector outputs (S2SS) ----------
 #define TESTBENCH_VECTORMATH_S2SS(name,in)                              \
   {                                                                     \
     XLAL_CHECK ( XLALVector##name##REAL4_GEN( xOutRef, xOutRef2, xIn, Ntrials ) == XLAL_SUCCESS, XLAL_EFUNC ); \
@@ -87,7 +122,7 @@
     XLAL_CHECK ( (maxRelerr <= (reltol)), XLAL_ETOL, "%s: relative error (%g) exceeds tolerance (%g)\n", #name "REAL4", maxRelerr, reltol ); \
   }
 
-
+// ----- test and benchmark operators with 2 REAL4 vector inputs to 1 REAL4 vector output (SS2S) ----------
 #define TESTBENCH_VECTORMATH_SS2S(name,in1,in2)                         \
   {                                                                     \
     XLAL_CHECK ( XLALVector##name##REAL4_GEN( xOutRef, in1, in2, Ntrials ) == XLAL_SUCCESS, XLAL_EFUNC ); \
@@ -110,7 +145,30 @@
     XLAL_CHECK ( (maxRelerr <= (reltol)), XLAL_ETOL, "%s: relative error (%g) exceeds tolerance (%g)\n", #name "REAL4", maxRelerr, reltol ); \
   }
 
+// ----- test and benchmark operators with 3 REAL4 vector inputs to 1 REAL4 vector output (SS2S) ----------
+#define TESTBENCH_VECTORMATH_SSS2S(name,in1,in2,in3)                    \
+  {                                                                     \
+    XLAL_CHECK ( XLALVector##name##REAL4_GEN( xOutRef, in1, in2, in3, Ntrials ) == XLAL_SUCCESS, XLAL_EFUNC ); \
+    tic = XLALGetCPUTime();                                             \
+    for (UINT4 l=0; l < Nruns; l ++ ) {                                 \
+      XLAL_CHECK ( XLALVector##name##REAL4( xOut, in1, in2, in3, Ntrials ) == XLAL_SUCCESS, XLAL_EFUNC ); \
+    }                                                                   \
+    toc = XLALGetCPUTime();                                             \
+    maxErr = maxRelerr = 0;                                             \
+    for ( UINT4 i = 0; i < Ntrials; i ++ )                              \
+    {                                                                   \
+      REAL4 err = fabsf ( xOut[i] - xOutRef[i] );                      \
+      REAL4 relerr = Relerr ( err, xOutRef[i] );                       \
+      maxErr    = fmaxf ( err, maxErr );                                \
+      maxRelerr = fmaxf ( relerr, maxRelerr );                          \
+    }                                                                   \
+    XLALPrintInfo ( "%-32s: %4.0f Mops/sec [maxErr = %7.2g (tol=%7.2g), maxRelerr = %7.2g (tol=%7.2g)]\n", \
+                    XLALVector##name##REAL4_name, (REAL8)Ntrials * Nruns / (toc - tic)/1e6, maxErr, (abstol), maxRelerr, (reltol) ); \
+    XLAL_CHECK ( (maxErr <= (abstol)), XLAL_ETOL, "%s: absolute error (%g) exceeds tolerance (%g)\n", #name "REAL4", maxErr, abstol ); \
+    XLAL_CHECK ( (maxRelerr <= (reltol)), XLAL_ETOL, "%s: relative error (%g) exceeds tolerance (%g)\n", #name "REAL4", maxRelerr, reltol ); \
+  }
 
+// ----- test and benchmark operators with 2 REAL4 vector inputs to 1 UINT4 scalar and 1 UINT4 vector output (SS2uU) ----------
 #define TESTBENCH_VECTORMATH_SS2uU(name,in1,in2)                        \
   {                                                                     \
     UINT4 xCount = 0, xCountRef = 0;                                    \
@@ -128,6 +186,25 @@
     XLALPrintInfo ( "%-32s: %4.0f Mops/sec\n", XLALVector##name##REAL4_name, (REAL8)Ntrials * Nruns / (toc - tic)/1e6 ); \
   }
 
+// ----- test and benchmark operators with 1 REAL8 vector input and 1 REAL8 scalar output (D2d) ----------
+#define TESTBENCH_VECTORMATH_D2d(name,in)                               \
+  {                                                                     \
+    REAL8 sOutRef, sOut;                                                \
+    XLAL_CHECK ( XLALVector##name##REAL8_GEN( &sOutRef, in, Ntrials ) == XLAL_SUCCESS, XLAL_EFUNC ); \
+    tic = XLALGetCPUTime();                                           \
+    for (UINT4 l=0; l < Nruns; l ++ ) {                                 \
+      XLAL_CHECK ( XLALVector##name##REAL8( &sOut, in, Ntrials ) == XLAL_SUCCESS, XLAL_EFUNC ); \
+    }                                                                   \
+    toc = XLALGetCPUTime();                                           \
+    maxErr = fabs( sOut - sOutRef );                                   \
+    maxRelerr = Relerrd( maxErr, sOutRef );                                \
+    XLALPrintInfo ( "%-32s: %4.0f Mops/sec [maxErr = %7.2g (tol=%7.2g), maxRelerr = %7.2g (tol=%7.2g)]\n", \
+                    XLALVector##name##REAL8_name, (REAL8)Ntrials * Nruns / (toc - tic)/1e6, maxErr, (abstol), maxRelerr, (reltol) ); \
+    XLAL_CHECK ( (maxErr <= (abstol)), XLAL_ETOL, "%s: absolute error (%g) exceeds tolerance (%g)\n", #name "REAL8", maxErr, abstol ); \
+    XLAL_CHECK ( (maxRelerr <= (reltol)), XLAL_ETOL, "%s: relative error (%g) exceeds tolerance (%g)\n", #name "REAL8", maxRelerr, reltol ); \
+  }
+
+// ----- test and benchmark operators with 2 REAL8 vector inputs to 1 REAL8 vector output (DD2D) ----------
 #define TESTBENCH_VECTORMATH_DD2D(name,in1,in2)                         \
   {                                                                     \
     XLAL_CHECK ( XLALVector##name##REAL8_GEN( xOutRefD, in1, in2, Ntrials ) == XLAL_SUCCESS, XLAL_EFUNC ); \
@@ -150,6 +227,30 @@
     XLAL_CHECK ( (maxRelerr <= (reltol)), XLAL_ETOL, "%s: relative error (%g) exceeds tolerance (%g)\n", #name "REAL8", maxRelerr, reltol ); \
   }
 
+// ----- test and benchmark operators with 3 REAL8 vector inputs to 1 REAL8 vector output (DD2D) ----------
+#define TESTBENCH_VECTORMATH_DDD2D(name,in1,in2,in3)                    \
+  {                                                                     \
+    XLAL_CHECK ( XLALVector##name##REAL8_GEN( xOutRefD, in1, in2, in3, Ntrials ) == XLAL_SUCCESS, XLAL_EFUNC ); \
+    tic = XLALGetCPUTime();                                             \
+    for (UINT4 l=0; l < Nruns; l ++ ) {                                 \
+      XLAL_CHECK ( XLALVector##name##REAL8( xOutD, in1, in2, in3, Ntrials ) == XLAL_SUCCESS, XLAL_EFUNC ); \
+    }                                                                   \
+    toc = XLALGetCPUTime();                                             \
+    maxErr = maxRelerr = 0;                                             \
+    for ( UINT4 i = 0; i < Ntrials; i ++ )                              \
+    {                                                                   \
+      REAL8 err = fabs ( xOutD[i] - xOutRefD[i] );                      \
+      REAL8 relerr = Relerrd ( err, xOutRefD[i] );                       \
+      maxErr    = fmax ( err, maxErr );                                \
+      maxRelerr = fmax ( relerr, maxRelerr );                          \
+    }                                                                   \
+    XLALPrintInfo ( "%-32s: %4.0f Mops/sec [maxErr = %7.2g (tol=%7.2g), maxRelerr = %7.2g (tol=%7.2g)]\n", \
+                    XLALVector##name##REAL8_name, (REAL8)Ntrials * Nruns / (toc - tic)/1e6, maxErr, (abstol), maxRelerr, (reltol) ); \
+    XLAL_CHECK ( (maxErr <= (abstol)), XLAL_ETOL, "%s: absolute error (%g) exceeds tolerance (%g)\n", #name "REAL8", maxErr, abstol ); \
+    XLAL_CHECK ( (maxRelerr <= (reltol)), XLAL_ETOL, "%s: relative error (%g) exceeds tolerance (%g)\n", #name "REAL8", maxRelerr, reltol ); \
+  }
+
+// ----- test and benchmark operators with 2 COMPLEX8 vector inputs to 1 COMPLEX8 vector output (CC2C) ----------
 #define TESTBENCH_VECTORMATH_CC2C(name,in1,in2)                         \
   {                                                                     \
     XLAL_CHECK ( XLALVector##name##COMPLEX8_GEN( xOutRefC, in1, in2, Ntrials ) == XLAL_SUCCESS, XLAL_EFUNC ); \
@@ -172,6 +273,29 @@
     XLAL_CHECK ( (maxRelerr <= (reltol)), XLAL_ETOL, "%s: relative error (%g) exceeds tolerance (%g)\n", #name "COMPLEX8", maxRelerr, reltol ); \
   }
 
+// ----- test and benchmark operators with 3 COMPLEX8 vector inputs to 1 COMPLEX8 vector output (CC2C) ----------
+#define TESTBENCH_VECTORMATH_CCC2C(name,in1,in2,in3)                    \
+  {                                                                     \
+    XLAL_CHECK ( XLALVector##name##COMPLEX8_GEN( xOutRefC, in1, in2, in3, Ntrials ) == XLAL_SUCCESS, XLAL_EFUNC ); \
+    tic = XLALGetCPUTime();                                             \
+    for (UINT4 l=0; l < Nruns; l ++ ) {                                 \
+      XLAL_CHECK ( XLALVector##name##COMPLEX8( xOutC, in1, in2, in3, Ntrials ) == XLAL_SUCCESS, XLAL_EFUNC ); \
+    }                                                                   \
+    toc = XLALGetCPUTime();                                             \
+    maxErr = maxRelerr = 0;                                             \
+    for ( UINT4 i = 0; i < Ntrials; i ++ )                              \
+    {                                                                   \
+      REAL4 err = cabsf ( xOutC[i] - xOutRefC[i] );                      \
+      REAL4 relerr = cRelerr ( err, xOutRefC[i] );                       \
+      maxErr    = fmaxf ( err, maxErr );                                \
+      maxRelerr = fmaxf ( relerr, maxRelerr );                          \
+    }                                                                   \
+    XLALPrintInfo ( "%-32s: %4.0f Mops/sec [maxErr = %7.2g (tol=%7.2g), maxRelerr = %7.2g (tol=%7.2g)]\n", \
+                    XLALVector##name##COMPLEX8_name, (REAL8)Ntrials * Nruns / (toc - tic)/1e6, maxErr, (abstol), maxRelerr, (reltol) ); \
+    XLAL_CHECK ( (maxErr <= (abstol)), XLAL_ETOL, "%s: absolute error (%g) exceeds tolerance (%g)\n", #name "COMPLEX8", maxErr, abstol ); \
+    XLAL_CHECK ( (maxRelerr <= (reltol)), XLAL_ETOL, "%s: relative error (%g) exceeds tolerance (%g)\n", #name "COMPLEX8", maxRelerr, reltol ); \
+  }
+
 // ----- test and benchmark operators with 1 REAL8 vector input and 1 REAL8 vector output (D2D) ----------
 #define TESTBENCH_VECTORMATH_D2D(name,in)                               \
   {                                                                     \
@@ -232,6 +356,17 @@ main ( int argc, char *argv[] )
   UINT4 Nruns = (UINT4)uvar->Nruns;
 
   UINT4 Ntrials = 1000000 + 7;
+
+  INT4Vector *xOutI4;
+  INT4Vector *xOutRefI4;
+  XLAL_CHECK ( ( xOutI4 = XLALCreateINT4Vector ( Ntrials )) != NULL, XLAL_EFUNC );
+  XLAL_CHECK ( ( xOutRefI4 = XLALCreateINT4Vector ( Ntrials )) != NULL, XLAL_EFUNC );
+
+  UINT4Vector *xOutU4;
+  UINT4Vector *xOutRefU4;
+  XLAL_CHECK ( ( xOutU4 = XLALCreateUINT4Vector ( Ntrials )) != NULL, XLAL_EFUNC );
+  XLAL_CHECK ( ( xOutRefU4 = XLALCreateUINT4Vector ( Ntrials )) != NULL, XLAL_EFUNC );
+
   REAL4VectorAligned *xIn_a, *xIn2_a, *xOut_a, *xOut2_a;
   XLAL_CHECK ( ( xIn_a   = XLALCreateREAL4VectorAligned ( Ntrials, uvar->inAlign )) != NULL, XLAL_EFUNC );
   XLAL_CHECK ( ( xIn2_a  = XLALCreateREAL4VectorAligned ( Ntrials, uvar->inAlign )) != NULL, XLAL_EFUNC );
@@ -249,11 +384,6 @@ main ( int argc, char *argv[] )
   REAL4 *xOutRef  = xOutRef_a->data;
   REAL4 *xOutRef2 = xOutRef2_a->data;
 
-  UINT4Vector *xOutU4;
-  UINT4Vector *xOutRefU4;
-  XLAL_CHECK ( ( xOutU4 = XLALCreateUINT4Vector ( Ntrials )) != NULL, XLAL_EFUNC );
-  XLAL_CHECK ( ( xOutRefU4 = XLALCreateUINT4Vector ( Ntrials )) != NULL, XLAL_EFUNC );
-
   REAL8VectorAligned *xInD_a, *xIn2D_a, *xOutD_a, *xOutRefD_a;
   XLAL_CHECK ( ( xInD_a   = XLALCreateREAL8VectorAligned ( Ntrials, uvar->inAlign )) != NULL, XLAL_EFUNC );
   XLAL_CHECK ( ( xIn2D_a  = XLALCreateREAL8VectorAligned ( Ntrials, uvar->inAlign )) != NULL, XLAL_EFUNC );
@@ -278,16 +408,26 @@ main ( int argc, char *argv[] )
   COMPLEX8 *xOutC     = xOutC_a->data;
   COMPLEX8 *xOutRefC  = xOutRefC_a->data;
 
-
   REAL8 tic, toc;
   REAL4 maxErr = 0, maxRelerr = 0;
   REAL4 abstol, reltol;
 
-  XLALPrintInfo ("Testing sin(x), cos(x) for x in [-1000, 1000]\n");
   for ( UINT4 i = 0; i < Ntrials; i ++ ) {
     xIn[i] = 2000 * ( frand() - 0.5 );
+    xInD[i] = xIn[i];
   }
   abstol = 2e-7, reltol = 1e-5;
+
+  XLALPrintInfo ("Testing (INT4) for x in [-1000, 1000]\n");
+  // ==================== (INT4) ====================
+  TESTBENCH_VECTORMATH_S2I(INT4From,xIn);
+
+  // ==================== SCALAR MAX ====================
+  XLALPrintInfo ("Testing scalar max for x in [-1000, 1000]\n");
+  TESTBENCH_VECTORMATH_S2s(ScalarMax,xIn);
+  TESTBENCH_VECTORMATH_D2d(ScalarMax,xInD);
+
+  XLALPrintInfo ("Testing sin(x), cos(x) for x in [-1000, 1000]\n");
   // ==================== SIN() ====================
   TESTBENCH_VECTORMATH_S2S(Sin,xIn);
 
@@ -314,7 +454,7 @@ main ( int argc, char *argv[] )
   for ( UINT4 i = 0; i < Ntrials; i ++ ) {
     xIn[i] = 10000.0f * frand() + 1e-6;
   } // for i < Ntrials
-  abstol = 2e-6, reltol = 2e-7;
+  abstol = 2e-6, reltol = 4e-7;
 
   TESTBENCH_VECTORMATH_S2S(Log,xIn);
 
@@ -341,6 +481,7 @@ main ( int argc, char *argv[] )
 
   TESTBENCH_VECTORMATH_SS2S(Shift,xIn[0],xIn2);
   TESTBENCH_VECTORMATH_SS2S(Scale,xIn[0],xIn2);
+  TESTBENCH_VECTORMATH_SSS2S(ScaleAdd,xIn[0],xIn,xIn2);
 
   TESTBENCH_VECTORMATH_DD2D(Add,xInD,xIn2D);
   TESTBENCH_VECTORMATH_DD2D(Sub,xInD,xIn2D);
@@ -349,12 +490,14 @@ main ( int argc, char *argv[] )
 
   TESTBENCH_VECTORMATH_DD2D(Shift,xInD[0],xIn2D);
   TESTBENCH_VECTORMATH_DD2D(Scale,xInD[0],xIn2D);
+  TESTBENCH_VECTORMATH_DDD2D(ScaleAdd,xInD[0],xInD,xIn2D);
 
   TESTBENCH_VECTORMATH_CC2C(Multiply,xInC,xIn2C);
   TESTBENCH_VECTORMATH_CC2C(Add,xInC,xIn2C);
 
   TESTBENCH_VECTORMATH_CC2C(Scale,xInC[0],xIn2C);
   TESTBENCH_VECTORMATH_CC2C(Shift,xInC[0],xIn2C);
+  TESTBENCH_VECTORMATH_CCC2C(ScaleAdd,xIn[0],xInC,xIn2C);
 
   // ==================== FIND ====================
   for ( UINT4 i = 0; i < Ntrials; i ++ ) {
@@ -370,6 +513,12 @@ main ( int argc, char *argv[] )
   XLALPrintInfo ("\n");
 
   // ---------- clean up memory ----------
+  XLALDestroyINT4Vector ( xOutI4 );
+  XLALDestroyINT4Vector ( xOutRefI4 );
+
+  XLALDestroyUINT4Vector ( xOutU4 );
+  XLALDestroyUINT4Vector ( xOutRefU4 );
+
   XLALDestroyREAL4VectorAligned ( xIn_a );
   XLALDestroyREAL4VectorAligned ( xIn2_a );
   XLALDestroyREAL4VectorAligned ( xOut_a );
@@ -378,9 +527,6 @@ main ( int argc, char *argv[] )
   XLALDestroyREAL4VectorAligned ( xOutRef_a );
   XLALDestroyREAL4VectorAligned ( xOutRef2_a );
 
-  XLALDestroyUINT4Vector ( xOutU4 );
-  XLALDestroyUINT4Vector ( xOutRefU4 );
-
   XLALDestroyREAL8VectorAligned ( xInD_a );
   XLALDestroyREAL8VectorAligned ( xIn2D_a );
   XLALDestroyREAL8VectorAligned ( xOutD_a );
